window.Gleam.initSearch([{"doc":"gleb128","title":"gleb128","content":"# GLEB128\n\n[![Package Version](https://img.shields.io/hexpm/v/gleb128)](https://hex.pm/packages/gleb128)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleb128/)\n[![Package License](https://img.shields.io/hexpm/l/gleb128)](https://hex.pm/packages/gleb128)\n[![Package Total Downloads Count](https://img.shields.io/hexpm/dt/gleb128)](https://hex.pm/packages/gleb128)\n[![Build Status](https://img.shields.io/github/actions/workflow/status/BrendoCosta/gleb128/test.yml)](https://hex.pm/packages/gleb128)\n[![Total Stars Count](https://img.shields.io/github/stars/BrendoCosta/gleb128)](https://hex.pm/packages/gleb128)\n\n## Description\n\nGLEB128 is a small Gleam library that provides functions for encoding and decoding LEB128 (Little Endian Base 128) integers. LEB128 is a variable-length code compression method used to store arbitrarily large integers in a small number of bytes. Notable use cases for LEB128 are in the DWARF debug file format and the WebAssembly&#39;s binary format.\n\n## Usage\n\n### Encoding\n\n```gleam\nimport gleam/io\nimport gleb128\n\npub fn main()\n{\n    let unsigned_encoded = gleb128.encode_unsigned(255)\n    let signed_encoded = gleb128.encode_signed(-255)\n\n    io.debug(unsigned_encoded)\n    io.debug(signed_encoded)\n}\n```\n\nShows the following in output:\n\n```console\nOk(&lt;&lt;255, 1&gt;&gt;)\n&lt;&lt;129, 126&gt;&gt;\n```\n\nWhich corresponds, in hexadecimal, respectively, to ``&lt;&lt;0xff, 0x01&gt;&gt;`` and ``&lt;&lt;0x81, 0x7e&gt;&gt;``.\n\n### Decoding\n\nThe decoding functions returns a ``Result`` wrapping a tuple containing the decoded value in its first position, followed by the count of bytes read in its second position.\n\n```gleam\nimport gleam/io\nimport gleb128\n\npub fn main()\n{\n    let unsigned_decoded = gleb128.decode_unsigned(&lt;&lt;255, 1, 4, 80, 64&gt;&gt;) // actually &lt;&lt;4, 80, 64&gt;&gt; will be ignored, so only the first two bytes will be read\n    let signed_decoded = gleb128.decode_signed(&lt;&lt;129, 126&gt;&gt;)\n\n    io.debug(unsigned_decoded)\n    io.debug(signed_decoded)\n}\n```\n\nShows the following in output:\n\n```console\nOk(#(255, 2))\nOk(#(-255, 2))\n```\n\n### Fast decoding\n\nThe ``fast_decode_unsigned`` and ``fast_decode_signed`` functions are optimized for decoding small LEB128 integers on 64-bit systems. Those functions will treat and process the data as a native integer when its length is less than or equal to 8 bytes (64 bits); otherwise, they will fallback to the default decoding functions.\n\nOn a Ryzen 5 5600G with 32 GB RAM, encoding and then decoding all numbers in the range from 0 to 100000000 with the default ``decode_signed`` took 50.10 seconds and used about 20 GB of memory. Repeating the test using ``fast_decode_signed`` reduced the elapsed time to 40.16 seconds and memory usage to about 14.5 GB. The ``fast_decode_unsigned`` function can be even faster when targeting Erlang, as it can use its stdlib&#39;s built-in ``binary:decode_unsigned/2`` function.\n\n## License\n\nGLEB128 source code is avaliable under the [MIT license](/LICENSE).\n","url":"index.html"},{"doc":"gleb128","title":"decode_signed","content":"pub fn decode_signed(\n  data: BitArray,\n) -&gt; Result(#(Int, Int), String)\n Decodes a bit array containing some LEB128 integer as an signed (positive or negative) native integer.\n \n Returns a tuple containing the decoded value in its first position, followed by the count of\n bytes read in its second position. Returns an error when the given data can&#39;t be decoded.\nSynonyms:\ngleb128.decode_signed\ngleb128 decode_signed","url":"gleb128.html#decode_signed"},{"doc":"gleb128","title":"decode_unsigned","content":"pub fn decode_unsigned(\n  data: BitArray,\n) -&gt; Result(#(Int, Int), String)\n Decodes a bit array containing some LEB128 integer as an unsigned (positive) native integer.\n \n Returns a tuple containing the decoded value in its first position, followed by the count of\n bytes read in its second position. Returns an error when the given data can&#39;t be decoded.\nSynonyms:\ngleb128.decode_unsigned\ngleb128 decode_unsigned","url":"gleb128.html#decode_unsigned"},{"doc":"gleb128","title":"encode_signed","content":"pub fn encode_signed(value: Int) -&gt; BitArray\n Encodes an signed (positive or negative) integer to a bit array containing its LEB128 representation.\nSynonyms:\ngleb128.encode_signed\ngleb128 encode_signed","url":"gleb128.html#encode_signed"},{"doc":"gleb128","title":"encode_unsigned","content":"pub fn encode_unsigned(value: Int) -&gt; Result(BitArray, String)\n Encodes an unsigned (positive) integer to a bit array containing its LEB128 representation.\n\n Returns an error when the given value to encode is negative.\nSynonyms:\ngleb128.encode_unsigned\ngleb128 encode_unsigned","url":"gleb128.html#encode_unsigned"},{"doc":"gleb128","title":"fast_decode_signed","content":"pub fn fast_decode_signed(\n  data: BitArray,\n) -&gt; Result(#(Int, Int), String)\n Decodes a bit array containing some LEB128 integer as an signed (positive or negative) native integer.\n When the length of the data is less than or equal to 8 bytes, this function will treat and process\n the data as an native integer, thus enabling a performance boost. It will fallback to the default\n decoding function when the length of the data is greater than 8 bytes.\n\n Returns a tuple containing the decoded value in its first position, followed by the count of\n bytes read in its second position. Returns an error when the given data can&#39;t be decoded.\nSynonyms:\ngleb128.fast_decode_signed\ngleb128 fast_decode_signed","url":"gleb128.html#fast_decode_signed"},{"doc":"gleb128","title":"fast_decode_unsigned","content":"pub fn fast_decode_unsigned(\n  data: BitArray,\n) -&gt; Result(#(Int, Int), String)\n Decodes a bit array containing some LEB128 integer as an unsigned (positive) native integer.\n When the length of the data is less than or equal to 8 bytes, this function will treat and process\n the data as an native integer, thus enabling a performance boost. It will fallback to the default\n decoding function when the length of the data is greater than 8 bytes.\n\n Returns a tuple containing the decoded value in its first position, followed by the count of\n bytes read in its second position. Returns an error when the given data can&#39;t be decoded.\nSynonyms:\ngleb128.fast_decode_unsigned\ngleb128 fast_decode_unsigned","url":"gleb128.html#fast_decode_unsigned"},{"doc":"gleb128","title":"gleb128","content":"","url":"gleb128.html"}]);